@@ -34,12 +34,12 @@ int Analitza::setPath(QString ar){
 	QDomElement docElem;
 	QFile file(ar);
 	if ( !file.open( IO_ReadOnly ) ){
-		err += i18n("Error while parsing: %1").arg(ar);
+		err += i18n("Error while parsing: %1<br />\n").arg(ar);
 		return -1;
 	}
 	if ( !doc.setContent( &file ) ) {
 		file.close();
-		err += i18n("Error while parsing: %1").arg(ar);
+		err += i18n("Error while parsing: %1<br/>").arg(ar);
 		return -2;
 	}
 	file.close();
@@ -53,7 +53,7 @@ int Analitza::setTextMML(QString str){
 	QDomDocument doc;
 	
 	if ( !doc.setContent(str) ) {
-		err += i18n("Error while parsing: %1").arg(str);
+		err += i18n("Error while parsing: %1<br />\n").arg(str);
 		return -1;
 	}
 	elem = doc.documentElement();
@@ -67,7 +67,7 @@ int Analitza::setText(QString op){
 	
 	if(a.error() != ""){
 		err = a.error();
-		return -1;
+		return -1;	
 	}
 	mmlexp=a.mathML();
 	
@@ -95,11 +95,13 @@ double Analitza::Calcula(){
 }
 
 double Analitza::evalua(QDomNode n){
-	QDomNode j;
+	QDomNode j, n_ini=n;
 	QDomElement e;
 	QString operador;
 	QValueList<double> nombres;
-	unsigned int fills=0;
+	double ret=.0;
+	int fills=0;
+	int sons=0;
 	
 	while( !n.isNull() ) {
 		e = n.toElement();
@@ -116,28 +118,53 @@ double Analitza::evalua(QDomNode n){
 			vars.modifica(s, j.toElement());
 		} else if(isOperador(e.tagName())) {
 			operador = e.tagName();
+			sons=isOperador(operador);
 		} else if (e.tagName() != "bvar")
-			err += i18n("The operator %1 hasn't been implemented").arg(e.tagName());
+			err += i18n("The operator <em>%1</em> hasn't been implemented<br />\n").arg(e.tagName());
+		if(e.tagName() == "sum" || e.tagName() == "product")
+			break;
 		fills++;
 		n = n.nextSibling();
 	}
 	
-	QDomDocument a;
-	QValueList<double>::iterator it = nombres.begin();
-	double ret = *it;
-	
-	if(fills>2)
-		it++;
-	
-	for(; it != nombres.end(); ++it)
-		ret = opera(ret,*it,operador, fills<=2?1:0);
-	
+	if(operador=="sum") ret =sum(n);
+	else if(fills-1==sons || (sons==-1 && fills>=3) || operador == "" || operador == "minus"){
+		QDomDocument a;
+		QValueList<double>::iterator it = nombres.begin();
+		ret = *it;
+		
+		if(fills>2)
+			it++;
+		for(; it != nombres.end(); ++it)
+			ret = opera(ret,*it,operador, fills<=2?1:0);
+	} else { 
+		if(sons==-1)
+			err += i18n("The <em>%1</em> operator, should have more than 1 parameter<p />").arg(operador);
+		else
+			err += i18n("Can't have %1 parameter with <em>%2</em> operator, it should have %3 parameter<p />").arg(fills-1).arg(operador).arg(sons);
+	}
 	return ret;
 }
 
+double Analitza::sum(QDomNode n){
+	double ret=.0;
+	QString var=bvar(n.parentNode())[0];
+	vars.rename(var, QString("%1_").arg(var)); //We save the var value
+	double ul=toNum(uplimit(n.parentNode()).toElement());
+	double dl=toNum(downlimit(n.parentNode()).toElement());
+	
+	for(double a=dl; a<ul; a+=1.){
+		vars.modifica(var, a);
+		ret=opera(ret, evalua(first_val(n.parentNode())),"plus",1);
+	}
+	vars.remove(var);
+	vars.rename(QString("%1_").arg(var), var); //We restore the var value
+	return ret;
+}
+
+
 double Analitza::opera(double res, double oper, QString op, int minus=0){
-	double a=res; 
-	double b=oper;
+	double a=res, b=oper;
 	bool boolean=false;
 	
 	if(op == "plus"){
@@ -271,23 +298,39 @@ double Analitza::opera(double res, double oper, QString op, int minus=0){
 	} else if(op=="root") {
 		a = (b==2.0)? sqrt(a) : pow(a, 1.0/b);
 	}else {
-		if(op!="") err += i18n("The operator %1 hasn't been implemented").arg(op);
+		if(op!="") err += i18n("The operator <em>%1</em> hasn't been implemented<br/>").arg(op);
 	}
 	
 	return a;
 }
 
-QStringList Analitza::lambda(QDomNode el){
+QStringList Analitza::bvar(QDomNode el){
 	QStringList lambdas;
-	
 	QDomNodeList lambs = el.toElement().elementsByTagName(QString("bvar"));
-	for(int i=0; i<lambs.count(); i++){
+	
+	for(unsigned int i=0; i<lambs.count(); i++)
 		lambdas << lambs.item(i).toElement().text();
-	}
 	
 	return lambdas;
 }
 
+QDomNode Analitza::uplimit(QDomNode el){
+	return el.toElement().elementsByTagName(QString("uplimit")).item(0).firstChild();
+}
+
+QDomNode Analitza::downlimit(QDomNode el){
+	return el.toElement().elementsByTagName(QString("downlimit")).item(0).firstChild();
+}
+
+QDomNode Analitza::first_val(QDomNode el){
+	QDomNode a=el.firstChild();
+	while(!a.isNull()){
+		if(a.toElement().tagName() == "apply" || a.toElement().tagName() == "cn"  || a.toElement().tagName() == "ci")
+			break;
+		a=a.nextSibling();
+	}
+	return a;
+}
 
 bool Analitza::isNum(QString s){
 	if(s=="true")
@@ -306,10 +349,13 @@ bool Analitza::isNum(QString s){
 double Analitza::toNum(QDomElement val){
 	double ret=0.0;
 	bool wrong;
-// 	cout << "<" << val.tagName() << ">" << val.text() << "</" << val.tagName() << ">" << endl;
+	
 	if(val.tagName()=="apply")
 		return evalua(val);
-	else if(val.tagName() == "cn"){ // a es un nombre
+	else if(val.tagName()=="ci") {
+		ret = toNum(vars.value(val.text(), &wrong)); //a is a var
+		if (wrong) err += i18n("The variable <em>%1</em> doesn't exist<br />\n").arg(val.text());
+	} else if(val.tagName() == "cn"){ // a is a number
 		if(val.attribute("type","real") == "real"){
 			ret= val.text().stripWhiteSpace().toDouble(&wrong); //TODO: Base on double not implemented
 		} else if(val.attribute("type") == "integer"){
@@ -329,14 +375,12 @@ double Analitza::toNum(QDomElement val){
 			else if (val.text() == "&infin;")	; //TODO: Not implemented  }
 			else if (val.text() == "&NaN;")		; //TODO: Not implemented  }*/
 		}
-	} else if(val.tagName()=="ci")	{	ret = toNum(vars.value(val.text(), &wrong));
-		if (wrong) err += i18n("The variable %1 doesn't exist\n").arg(val.text());
 	} else if(val.tagName()=="true")		ret = 1.0;
 	else if(val.tagName()=="false")		ret = 0.0;
 	else if(val.tagName()=="pi")		ret = 3.1415926535897932384626433;
 	else if(val.tagName()=="exponentiale")	ret = exp(1.);
 	else if(val.tagName()=="eulergamma")	ret = 0.5772156649;
-// 	qDebug("el numero premiado es: %f", ret);
+	
 	return ret;
 }
 
@@ -346,10 +390,12 @@ int Analitza::isOperador(QString e){
 			e=="min" ||
 			e=="max" ||
 			e=="and" ||
-			e=="or" ||
+			e=="or"  ||
 			e=="xor" ||
 			e=="gcd" ||
-			e=="lcm") {
+			e=="lcm" ||
+			e=="max" ||
+			e=="min") {
 		return -1;
 	} else if (	e=="quotient" ||
 			e=="power" ||
@@ -364,46 +410,48 @@ int Analitza::isOperador(QString e){
 			e=="implies" ||
 			e=="root"){
 		return 2;
-	} else if (	e=="factorial" || 
-			e=="abs" || 
-			e=="sin" || 
-			e=="cos" || 
-			e=="tan" || 
-			e=="sec" || 
-			e=="csc" || 
+	} else if (	e=="factorial" ||
+			e=="abs" ||
+			e=="sin" ||
+			e=="cos" ||
+			e=="tan" ||
+			e=="sec" ||
+			e=="csc" ||
 			e=="cot" ||
-			e=="sinh" || 
-			e=="cosh" || 
-			e=="tanh" || 
-			e=="sech" || 
-			e=="csch" || 
-			e=="coth" || 
-			e=="arcsin" || 
-			e=="arccos" || 
-			e=="arctan" || 
-			e=="arccos" || 
-			e=="arccot" || 
-			e=="arcoth" || 
-			e=="arccosh" || 
-			e=="arccsc"  || 
-			e=="arccsch" || 
-			e=="arcsec"  || 
-			e=="arcsech" || 
-			e=="arcsinh" || 
-			e=="arctanh" || 
-			e=="exp" || 
-			e=="ln" || 
-			e=="log"||
+			e=="sinh" ||
+			e=="cosh" ||
+			e=="tanh" ||
+			e=="sech" ||
+			e=="csch" ||
+			e=="coth" ||
+			e=="arcsin" ||
+			e=="arccos" ||
+			e=="arctan" ||
+			e=="arccos" ||
+			e=="arccot" ||
+			e=="arcoth" ||
+			e=="arccosh" ||
+			e=="arccsc"  ||
+			e=="arccsch" ||
+			e=="arcsec"  ||
+			e=="arcsech" ||
+			e=="arcsinh" ||
+			e=="arctanh" ||
+			e=="exp" ||
+			e=="ln" ||
+			e=="log" ||
 			e=="not" ||
-			e=="factorial" || 
-			e=="minus" || 
-			e=="abs" || 
+			e=="factorial" ||
+			e=="minus" ||
+			e=="abs" ||
 			e=="conjugate" ||
-			e=="arg" || 
-			e=="real" || 
-			e=="imaginary" || 
-			e=="floor" || 
-			e=="ceiling") {
+			e=="arg" ||
+			e=="real" ||
+			e=="imaginary" ||
+			e=="floor" ||
+			e=="ceiling" ||
+			e=="sum" ||
+			e=="product") {
 		return 1;
 	}
 	
@@ -411,7 +459,7 @@ int Analitza::isOperador(QString e){
 }
 
 QStringList Analitza::lambda(){
-	return lambda(elem.firstChild());
+	return bvar(elem.firstChild());
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -428,7 +476,7 @@ QStringList Analitza::lambda(){
 ////////////////////////////////////////////////////////////////////////
 
 QString Analitza::toString(){
-	return QString("<math>%1</math>").arg(str(elem.firstChild()));
+	return str(elem.firstChild());
 }
 
 QString Analitza::get_op(QDomNode n){
@@ -473,27 +521,28 @@ QString Analitza::str(QDomNode n){
 		e = n.toElement();
 		if(e.tagName() == "apply" || e.tagName() == ""){
 			j = e.firstChild();
-			//It is not necessary to have everything between ()
-			if(toOpId(k) <= toOpId(j) || isOperador(get_op(j)) == 1)
+			if(toOpId(k) <= toOpId(j) || isOperador(get_op(j)) == 1) //It is not necessary to have everything between ()
 				nombres.append(str(j));
 			else
 				nombres.append("(" + str(j) + ")");
-		} else if (e.tagName() == "cn" ||e.tagName() == "ci") {
-			nombres.append("<mn>" + e.text() + "</mn>");
+		} else if (e.tagName() == "cn"){
+			nombres.append(QString("<span class='num'>%1</span>").arg(e.text()));
+		} else if (e.tagName() == "ci") {
+			nombres.append(QString("<span class='var'>%1</span>").arg(e.text()));
 		} else if(isNum(e.tagName())){
-			nombres.append("<mn>" + e.tagName() + "</mn>");
+			nombres.append(e.tagName());
 		} else if (isOperador(e.tagName())) {
 			operador = e.tagName();
 		} else if (e.tagName()=="declare") {
 			j = e.firstChild(); //A <ci>
 			nombres.append(j.toElement().text());
 			j = j.nextSibling(); //Value to save
-			nombres.append(j.toElement().text());
+			nombres.append(str(j));
 			operador = "declare";
 			fills=3;
 		} else if (e.tagName()=="lambda"){
 			j = e.firstChild();
-			QStringList lambdas = lambda(j);
+			QStringList lambdas = bvar(j);
 			for (QStringList::Iterator it = lambdas.begin(); it != lambdas.end(); ++it){
 				nombres.append(*it);
 				j=j.nextSibling();
@@ -505,6 +554,7 @@ QString Analitza::str(QDomNode n){
 		fills++;
 		n= n.nextSibling();
 	}
+	
 	QValueList<QString>::iterator it = nombres.begin();
 	QString r= *it;
 	
@@ -512,12 +562,41 @@ QString Analitza::str(QDomNode n){
 		it++;
 		
 	for(; it != nombres.end(); ++it)
-		r = escriu(r,*it,operador, fills<=2?1:0);
+		r = escriuS(r,*it,operador, fills<=2?1:0);
 	
 	return r;
 }
 
-QString Analitza::escriu(QString res, QString oper, QString op, int unari=0){
+QString Analitza::escriuS(QString res, QString oper, QString op, int unari=0){
+	res  =  res.stripWhiteSpace();
+	oper = oper.stripWhiteSpace();
+	
+	QString r=res;
+	if(op == "plus"){
+		r = QString("%1<span class='op'>+</span>%2").arg(res).arg(oper);
+	}else if(op =="times"){
+		r = QString("%1<span class='op'>*</span>%2").arg(res).arg(oper);
+	}else if(op=="quotient"){
+		r = QString("%1<span class='op'>/</span>%2").arg(res).arg(oper);
+	}else if(op=="minus"){
+		r = unari ? QString("<span class='op'>-</span>%1").arg(res) : QString("%1<span class='op'>-</span>%2").arg(res).arg(oper);
+	}else if(op=="power"){
+		r = QString("%1<span class='op'>^</span>%2").arg(res).arg(oper);
+	} else if(op=="declare"){
+		r = QString("%1<span class='op'>:=</span>%2").arg(res).arg(oper);
+	} else if(op=="lambda"){
+		r = QString("%1<span class='op'>-></span>%2").arg(res).arg(oper);
+	} else if(op != "") {
+		if(unari)
+			r = QString("<span class='func'>%1</span><span class='op'>(</span>%2<span class='op'>)</span>").arg(op).arg(res);
+		else
+			r = QString("<span class='func'>%1</span><span class='op'>(</span>%2,%3<span class='op'>)</span>").arg(op).arg(res).arg(oper);
+		//qDebug("The operator: <%s /> is not implemented.", operador.latin1());
+	}
+	return r;
+}
+
+QString Analitza::escriuMMLP(QString res, QString oper, QString op, int unari=0){
 	res  =  res.stripWhiteSpace();
 	oper = oper.stripWhiteSpace();
 	
@@ -572,13 +651,13 @@ QString Analitza::escriu(QString res, QString oper, QString op, int unari=0){
 	} else if(op=="csch"){
 		r = "cosech(" + res + ")";
 	} else if(op=="coth"){
-		r = "cotgh(" + res + ")";
+		r = "coth(" + res + ")";
 	} else if(op=="arcsin"){
 		r = "arcsin(" + res + ")";
 	} else if(op=="arccos"){
 		r = "arccos(" + res + ")";
 	} else if(op=="arctan"){
-		r = "arctg(" + res + ")";
+		r = "arctan(" + res + ")";
 	} else if(op=="arccos"){
 		r = "arccos(" + res + ")";
 	} else if(op=="arccot"){
@@ -661,3 +740,19 @@ QString Analitza::treu_tags(QString in){
 	}
 	return out;
 }
+
+void print_dom(QDomNode in, int ind=0){
+	QString a="";
+	if(ind >100){
+		qDebug("g0g0g0");
+		return;
+	}
+	for(int i=0; i<ind; i++)
+		a+="--------";
+	qDebug("%s%s(%s)", a.ascii(), in.toElement().tagName().ascii(), in.toElement().text().ascii());
+	if(in.childNodes().length()==1)
+		return;
+	for(unsigned int i=0 ; i<in.childNodes().length(); i++){
+		print_dom(in.childNodes().item(i), ind+1);
+	}
+}
