@@ -17,10 +17,11 @@ class Variables{
 		QValueList<struct VARIABLE> vars;
 	public:
 		Variables();
-		void afegeix(QString id, QDomElement val);
-		void treu(QString id, bool *ex=0);
-		void modifica(QString id, QDomElement new_val);
-		void modifica(QString id, double new_val);
+		bool add(QString id, QDomElement val);
+		bool remove(QString id);
+		bool modifica(QString id, QDomElement new_val);
+		bool modifica(QString id, double new_val);
+		bool rename(QString id, QString newId);
 		QDomElement value(QString id, bool *ex=NULL);
 		QStringList getNoms();
 		bool isvar(QDomElement e);
